// Configured for spectrum shield
//
// For Dual MSGEQ7
// Analogue 0 = Left Channel
// Analogue 1 = Right Channel
// Analogue 2 = LDR
// D4 = Digital O/P Strobe
// D5 = Digital O/P Reset
//
// MAX7219 Control. Assumes two 8x8 LED Matrices
//
// D10 = Digital O/P CS Pin
// D11 = Digital O/P Clock Pin
// D12 = Digital O/P Data Pin
//

#include <LedControl.h>

const int numDevices = 2;      // number of MAX7219s used
const int dataPin = 12;
const int clkPin = 11;
const int csPin = 10;
LedControl lc=LedControl(dataPin,clkPin,csPin,numDevices);

int analogueInputLeftChannel = 0; // read from multiplexer using analog input 0, Left Channel
int analogueInputRightChannel = 1; // read from multiplexer using analog input 1, Right Channel
int strobePin = 4; // strobe is attached to digital pin 4
int resetPin = 5; // reset is attached to digital pin 5
int spectrumValue1[7]; // to hold a2d values
int spectrumValue2[7]; // to hold a2d values

int analogueInputLDR = 5;
byte tempIntensity = 1;

byte tempLeftChannel = 0;
byte tempRightChannel = 0;
unsigned long tempLeftChannela = 0;
unsigned long tempRightChannela = 0;
unsigned long sumLeftChannel = 0;
unsigned long sumRightChannel = 0;
unsigned long leftChannelAverageIndex = 0;
unsigned long rightChannelAverageIndex = 0;
unsigned long rollingAverageLeftChannelOld = 0;
unsigned long rollingAverageRightChannelOld = 0;
unsigned long rollingAverageLeftChannelNew = 0;
unsigned long rollingAverageRightChannelNew = 0;
#define NORMAL_PASS_COUNT 10  // The number off passes through the loop before an average is updated
#define SILENT_PASS_COUNT 30 // How long the dot level persists after audio is removed. Makes the L & R drop slowly after audio is removed.
byte passCount = 0;
byte passCountLimit = NORMAL_PASS_COUNT; 
#define ADC_UPPER_LIMIT 900    // Max upper limit 1023
#define SILENT_CHANNEL_NOISE_SUPPRESSION 150  // stops spurious noise from setting off the display with no audio applied
#define DIVISION_FACTOR 6 // dividing by 4 makes averaging more sensitive
#define LDR_UPPER_LIMIT 1000
#define LDR_LOWER_LIMIT 0

byte movingdot[] = {
      B00000000,
      B10000000,
      B01000000,
      B00100000,
      B00010000,
      B00001000,
      B00000100,
      B00000010,
      B00000001
};

byte bargraph[] = {
      B00000000,
      B10000000,
      B11000000,
      B11100000,
      B11110000,
      B11111000,
      B11111100,
      B11111110,
      B11111111
};


const int leftLEDMatrix = 0;
const int rightLEDMatrix = 1;

void setup(){
   pinMode(analogueInputLeftChannel, INPUT);
   pinMode(analogueInputRightChannel, INPUT);
   pinMode(analogueInputLDR, INPUT);
   pinMode(strobePin, OUTPUT);
   pinMode(resetPin, OUTPUT);
   analogReference(DEFAULT);

   digitalWrite(resetPin, LOW);
   digitalWrite(strobePin, HIGH);
   for (int x=0; x<numDevices; x++){
        lc.shutdown(x,false);       //The MAX72XX is in power-saving mode on startup
        lc.setIntensity(x,tempIntensity);       // Set the brightness to default value
        lc.clearDisplay(x);         // and clear the display
    }
}


void loop(){ 
     tempIntensity = (byte) map(analogRead(analogueInputLDR), LDR_LOWER_LIMIT,LDR_UPPER_LIMIT,15,0);
     for (int x=0; x<numDevices; x++)
       lc.setIntensity(x,tempIntensity);       // Set the brightness from LDR value

     digitalWrite(resetPin, HIGH);
     delayMicroseconds(1); // to allow the output to settle
     digitalWrite(resetPin, LOW);
     delayMicroseconds(72); // to allow the output to settle

     sumLeftChannel = 0;
     sumRightChannel = 0;
     for (int i = 0; i < 7; i++)
     {
       digitalWrite(strobePin, LOW);
       delayMicroseconds(36); // to allow the output to settle
       
       tempLeftChannela = (unsigned long) analogRead(analogueInputLeftChannel);
       if (tempLeftChannela < SILENT_CHANNEL_NOISE_SUPPRESSION) tempLeftChannela = 0;
       tempLeftChannel = (byte) map(tempLeftChannela,0,ADC_UPPER_LIMIT,0,8);
       sumLeftChannel += tempLeftChannel;

       tempRightChannela = (unsigned long) analogRead(analogueInputRightChannel);
       if (tempRightChannela < SILENT_CHANNEL_NOISE_SUPPRESSION) tempRightChannela = 0;
       tempRightChannel = (byte) map(tempRightChannela,0,ADC_UPPER_LIMIT,0,8);
       sumRightChannel += tempRightChannel;
       lc.setRow(rightLEDMatrix,i,bargraph[tempRightChannel]); // In order
       lc.setRow(leftLEDMatrix,7-i,bargraph[tempLeftChannel]); // Reverse order
       digitalWrite(strobePin, HIGH);
       delayMicroseconds(36); // to allow the output to settle
     }
          
      // Detects when audio is not present and increases the number of passes in the loop before the moving dot is dropped
      if ((rollingAverageLeftChannelOld == rollingAverageLeftChannelNew) &&
          (rollingAverageRightChannelOld == rollingAverageRightChannelNew) &&
          (rollingAverageLeftChannelNew == 0) &&
          (rollingAverageRightChannelNew == 0))
        passCountLimit = SILENT_PASS_COUNT;
      else
        passCountLimit = NORMAL_PASS_COUNT;

     if (passCount > passCountLimit) 
     {
         passCount = 0;

         rollingAverageLeftChannelNew = map(sumLeftChannel/DIVISION_FACTOR,0,8,0,8);

         if ((rollingAverageLeftChannelNew == rollingAverageLeftChannelOld) && 
             (rollingAverageLeftChannelNew == 0) && 
             (leftChannelAverageIndex > 0))
         {
            if (leftChannelAverageIndex > 0) leftChannelAverageIndex--;
            lc.setRow(leftLEDMatrix,0,movingdot[leftChannelAverageIndex]); 
         }

         if (rollingAverageLeftChannelOld != rollingAverageLeftChannelNew) 
         {
            if (rollingAverageLeftChannelOld < rollingAverageLeftChannelNew) 
            {
              if (leftChannelAverageIndex < 8) leftChannelAverageIndex++;
            } else {
                 if (rollingAverageLeftChannelOld > rollingAverageLeftChannelNew) 
                 {
                   if (leftChannelAverageIndex > 0) leftChannelAverageIndex--;
                 } else {
                   // The same do nothing
                 }
            }
            lc.setRow(leftLEDMatrix,0,movingdot[leftChannelAverageIndex]); 
            rollingAverageLeftChannelOld = rollingAverageLeftChannelNew;
         }

         rollingAverageRightChannelNew = map(sumRightChannel/DIVISION_FACTOR,0,8,0,8);

         if ((rollingAverageRightChannelNew == rollingAverageRightChannelOld) && 
             (rollingAverageRightChannelNew == 0) && 
             (rightChannelAverageIndex > 0))
         {
            if (rightChannelAverageIndex > 0) rightChannelAverageIndex--;
            lc.setRow(rightLEDMatrix,7,movingdot[rightChannelAverageIndex]); 
         }

         if (rollingAverageRightChannelOld != rollingAverageRightChannelNew) 
         {
            if (rollingAverageRightChannelOld < rollingAverageRightChannelNew) 
            {
              if (rightChannelAverageIndex < 8) rightChannelAverageIndex++;
            } else {
                 if (rollingAverageRightChannelOld > rollingAverageRightChannelNew) 
                 {
                    if (rightChannelAverageIndex > 0) rightChannelAverageIndex--;
                 } else {
                     // The same do nothing
                 }
            }
            lc.setRow(rightLEDMatrix,7,movingdot[rightChannelAverageIndex]); 
            rollingAverageRightChannelOld = rollingAverageRightChannelNew;
         }
     } else {
         passCount++;     
     }
}



